import {
  emitHelperId,
  emit_helper_default,
  helperPrefix,
  transformDefineModel,
  useVmodelHelperId,
  use_vmodel_default
} from "./chunk-RWD5QG6M.mjs";

// src/index.ts
import { createUnplugin } from "unplugin";
import {
  REGEX_SETUP_SFC,
  REGEX_VUE_SFC,
  REGEX_VUE_SUB,
  createFilter,
  detectVueVersion,
  normalizePath
} from "@vue-macros/common";
function resolveOption(options, framework) {
  const version = options.version || detectVueVersion();
  return {
    include: [REGEX_VUE_SFC, REGEX_SETUP_SFC].concat(
      version === 2 && framework === "webpack" ? REGEX_VUE_SUB : []
    ),
    unified: true,
    ...options,
    version
  };
}
var name = "unplugin-vue-define-model";
var src_default = createUnplugin(
  (userOptions = {}, { framework }) => {
    const options = resolveOption(userOptions, framework);
    const filter = createFilter(options);
    return {
      name,
      enforce: "pre",
      resolveId(id) {
        if (id.startsWith(helperPrefix))
          return id;
      },
      loadInclude(id) {
        return id.startsWith(helperPrefix);
      },
      load(_id) {
        const id = normalizePath(_id);
        if (id === emitHelperId)
          return emit_helper_default;
        else if (id === useVmodelHelperId)
          return use_vmodel_default;
      },
      transformInclude(id) {
        return filter(id);
      },
      transform(code, id) {
        try {
          return transformDefineModel(
            code,
            id,
            options.version,
            options.unified
          );
        } catch (err) {
          this.error(`${name} ${err}`);
        }
      }
    };
  }
);

export {
  src_default
};
